#ifndef Application_h
#define Application_h

#ifndef rti_me_cpp_hxx
#include "rti_me_cpp.hxx"
#endif


/* Define USE_SECURITY to register the RTI Security Plugin factories
 * and enable them in the Application's DomainParticipant
 */
#define USE_SECURITY

/* Define USE_RELIABLE_QOS to use reliable
 * reliability; otherwise, best-effort is used
 */
#define USE_RELIABLE_QOS

/* Define USE_SAMPLE_FILTER to filter samples
 * using call-backs on the DataReader's listener*/

//#define USE_SAMPLE_FILTER

/* Define FILTER_ON_DESERIALIZE to enable
 * filtering on call-back on_before_sample_deserialize;
 * otherwise use call-back on_before_sample_commit
 * by default. */

//#define FILTER_ON_DESERIALIZE

extern const char *ca_string;

extern const char *subscriber_string;

extern const char *subscriber_key_string;

extern const char *governance_string;

extern const char *permissions_subscriber_string;

extern const char *publisher_string;

extern const char *publisher_key_string;

extern const char *permissions_publisher_string;


/* Filenames of required cryptographic material and configuration files */
class ApplicationSecMaterial
{
public:

    const char *ca_id_cert;
    const char *ca_perm_cert;
    const char *peer_key;
    const char *peer_cert;
    const char *xml_gov;
    const char *xml_perm;


    ApplicationSecMaterial(const char *ca_id_cert,
                           const char *ca_perm_cert,
                           const char *peer_key,
                           const char *peer_cert,
                           const char *xml_gov,
                           const char *xml_perm);
    ApplicationSecMaterial();
    ~ApplicationSecMaterial();

    void print();

    bool is_valid();
};

extern struct ApplicationSecMaterial PEER_MATERIAL_DEFAULT;

class Application
{
public:
    DDSDomainParticipant *participant; 
    DDSTopic *topic; 
    char topic_name[255];
    char type_name[255];
    DDS_Long sleep_time;
    DDS_Long count;
    ApplicationSecMaterial *sec_material;

    static void help(char *appname);

    DDS_ReturnCode_t initialize(const char *local_participant_name,
                                const char *remote_participant_name,
                                DDS_Long domain_id, 
                                char *udp_intf, 
                                char *peer, 
                                DDS_Long sleep_time, 
                                DDS_Long count,
                                ApplicationSecMaterial *sec_material);

    DDS_ReturnCode_t enable();

    Application();
    ~Application();

#ifdef USE_SECURITY
private:
    DDS_ReturnCode_t configure_security(RTRegistry *registry,
                                        DDS_DomainParticipantQos &dp_qos,
                                        ApplicationSecMaterial *sec_material);
#endif /* USE_SECURITY */
};

#endif
